"use strict";

/* eslint-disable no-console */

import React, { memo, useCallback, useEffect, useState } from 'react';
import { ScrollView, StyleSheet, Text, View } from 'react-native';
import { useShiki } from './SyntaxContext';
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
const DEBUG = true;
function log(...args) {
  if (DEBUG) console.log('[SyntaxHighlighter]', ...args);
}
const styles = StyleSheet.create({
  container: {
    flex: 1
  },
  line: {
    flexDirection: 'row',
    flexWrap: 'wrap'
  }
});
export const SyntaxHighlighter = /*#__PURE__*/memo(({
  code,
  language,
  theme = 'github-dark',
  fontSize = 14
}) => {
  const {
    highlighter,
    isReady
  } = useShiki();
  const [tokens, setTokens] = useState([]);
  const [error, setError] = useState(null);
  const highlight = useCallback(async () => {
    if (!highlighter || !isReady) return;
    try {
      const loadedLangs = highlighter.getLoadedLanguages();
      const normalizedLang = language.toLowerCase();
      const effectiveLang = loadedLangs.includes(normalizedLang) ? normalizedLang : 'text';

      // Add options to ensure proper tokenization
      const results = highlighter.codeToTokens(code, {
        lang: effectiveLang,
        theme: typeof theme === 'string' ? theme : theme?.name || 'default',
        includeExplanation: true
      });
      log('Raw tokenization result:', results);

      // Process tokens to ensure proper granularity
      const processedTokens = results.tokens.map(line => {
        // Split any tokens that contain multiple semantic parts
        return line.flatMap(token => {
          const parts = token.content.split(/([{}().,;=+\-*/<>])/g);
          if (parts.length <= 1) return [token];
          return parts.filter(Boolean).map(part => ({
            ...token,
            content: part,
            // Preserve the original token's styling
            color: token.color,
            fontStyle: token.fontStyle,
            explanation: token.explanation
          }));
        });
      });
      log('Processed tokens:', {
        lines: processedTokens.length,
        tokensPerLine: processedTokens.map(line => line.length),
        firstLine: processedTokens[0]?.map(t => ({
          content: t.content,
          color: t.color,
          scopes: t.explanation?.[0]?.scopes
        }))
      });
      setTokens(processedTokens);
      setError(null);
    } catch (err) {
      console.error('Highlighting error:', err);
      setError(err instanceof Error ? err.message : 'Failed to highlight code');
    }
  }, [code, language, theme, highlighter, isReady]);
  useEffect(() => {
    highlight();
  }, [highlight]);
  useEffect(() => {
    if (tokens.length > 0) {
      log('Rendering tokens:', {
        lineCount: tokens.length,
        tokensPerLine: tokens.map(line => line.length),
        firstLine: tokens[0]?.map(t => ({
          content: t.content,
          color: t.color,
          scopes: t.explanation?.[0]?.scopes
        }))
      });
    }
  }, [tokens]);
  if (!isReady) {
    return /*#__PURE__*/_jsx(View, {
      style: styles.container,
      children: /*#__PURE__*/_jsx(Text, {
        style: {
          fontSize
        },
        children: code
      })
    });
  }
  if (error) {
    return /*#__PURE__*/_jsxs(View, {
      style: styles.container,
      children: [/*#__PURE__*/_jsx(Text, {
        style: {
          color: 'red'
        },
        children: error
      }), /*#__PURE__*/_jsx(Text, {
        style: {
          fontSize
        },
        children: code
      })]
    });
  }
  return /*#__PURE__*/_jsx(ScrollView, {
    horizontal: true,
    style: styles.container,
    contentContainerStyle: {
      padding: 10
    },
    children: /*#__PURE__*/_jsx(View, {
      children: tokens.map((line, i) => /*#__PURE__*/_jsxs(View, {
        style: styles.line,
        children: [line.map((token, j) => /*#__PURE__*/_jsx(Text, {
          style: {
            color: token.color || '#e1e4e8',
            fontSize,
            ...(token.fontStyle && token.fontStyle & 1 && {
              fontStyle: 'italic'
            }),
            ...(token.fontStyle && token.fontStyle & 2 && {
              fontWeight: 'bold'
            }),
            backgroundColor: token.bgColor
          },
          children: token.content
        }, `${i}-${j}`)), /*#__PURE__*/_jsx(Text, {
          style: {
            color: '#e1e4e8',
            fontSize
          },
          children: '\n'
        })]
      }, i))
    })
  });
});
//# sourceMappingURL=SyntaxHighlighter.js.map