/* eslint-disable no-console */
import type { HighlighterCore, LanguageRegistration, ThemeRegistrationAny } from '@shikijs/types'
import { createHighlighterCore } from '@shikijs/core'

import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import React, { createContext, useContext, useEffect, useMemo, useState } from 'react'

const DEBUG = true
function log(...args: any[]) {
  if (DEBUG)
    console.log('[ShikiProvider]', ...args)
}

interface ShikiContextValue {
  highlighter: HighlighterCore | null
  isReady: boolean
  error: Error | null
}

interface ShikiProviderProps {
  children: React.ReactNode
  initialLanguages: LanguageRegistration[]
  initialThemes: ThemeRegistrationAny[]
}

const ShikiContext = createContext<ShikiContextValue | null>(null)

export function ShikiProvider({ children, initialLanguages, initialThemes }: ShikiProviderProps) {
  const [highlighter, setHighlighter] = useState<HighlighterCore | null>(null)
  const [isReady, setIsReady] = useState(false)
  const [error, setError] = useState<Error | null>(null)

  useEffect(() => {
    const init = async () => {
      try {
        log('Starting highlighter initialization')
        // Fix language logging by accessing the id/name properly
        log('Languages:', initialLanguages.map(l => l.name || l.scopeName))
        log('Themes:', initialThemes.map(t => t.name))

        const engine = createJavaScriptRegexEngine({
          forgiving: false, // Change to false to get more detailed errors
          cache: new Map(),
        })

        const instance = await createHighlighterCore({
          langs: initialLanguages,
          themes: initialThemes,
          engine,
        })

        // Test with more complex code to verify tokenization
        const testCode = `
    function test(x: number): string {
      return x.toString();
    }
    `
        const test = instance.codeToTokens(testCode.trim(), {
          lang: 'typescript',
          theme: 'github-dark',
          includeExplanation: true,
        })

        log('Tokenization test:', {
          lines: test.tokens.length,
          tokens: test.tokens.map(line =>
            line.map(token => ({
              content: token.content,
              color: token.color,
              scopes: token.explanation?.[0]?.scopes,
            })),
          ),
        })

        setHighlighter(instance)
        setIsReady(true)
      }
      catch (err) {
        console.error('Failed to initialize highlighter:', err)
        setError(err instanceof Error ? err : new Error('Failed to initialize highlighter'))
      }
    }

    init()
  }, [])

  const value = useMemo(
    () => ({
      highlighter,
      isReady,
      error,
    }),
    [highlighter, isReady, error],
  )

  return <ShikiContext.Provider value={value}>{children}</ShikiContext.Provider>
}

export function useShiki() {
  const context = useContext(ShikiContext)
  if (!context) {
    throw new Error('useShiki must be used within a ShikiProvider')
  }
  return context
}
